% Program: demo_4_2
%
%       Created for the book:                                                             
%   Ljiljana D. Milic,
%      Multirate Filtering for Digital Signal Processing: MATLAB Applications                     
%      Information Science Reference, Hershey, New York,  2009
%
% Structure simulation of decimator, Figure 4.8.
% Structure simulation of interpolator, Figure 4.10.
clear all, close all

% STEP 1
% Lowpass FIR filter design
h = firpm(59,[0,400/5000,1000/5000,1],[1,1,0,0]); % Computing the filter coefficients
[H,F1] = freqz(h,1,1024,10000); % Computing the signal spectrum
figure (1)
subplot(4,1,1), plot(F1,20*log10(abs(H))), legend('Filter magnitude response'), ylabel('Gain, dB')
axis([0,5000,-80,5])

% STEP 2
% generating the input signal
n = 0:510;  % Time index
x = cos(2*pi*200*n/10000)+cos(2*pi*400*n/10000)+0.7*cos(2*pi*3800*n/10000)+randn(size(n));
[X,F1] = freqz(x,1,1024,10000); % Computing the spectrum of the input signal
subplot(4,1,2)
plot(F1,abs(X)), legend('Spectrum: input signal'), ylabel('|X(F)|')
axis([0,5000,0,300])
% STEP 3
% Polyphase decomposition
E = reshape(h,5,length(h)/5);

% STEP 4
% Initial states in polyphase components
vk_i = zeros(size(E));
% Decimation
rot = 0; % Initial rotator position
ydec = [];
for n=1:length(x)
xn = x(n);
vk = vk_i(rot+1,:); ek = E(rot+1,:);
vk = [xn,vk(1:length(vk)-1)]; yn = sum(vk.*ek); % filtering
vk_i(rot+1,:)=vk;
if n==1, yy=0; else, end
if rot == 0 ss = 4; yy = yy+yn; ydec = [ydec,yy]; else end
if rot == 1 ss = 0; yy = yy+yn; else end
if rot == 2 ss = 1; yy = yy+yn; else end
if rot == 3 ss = 2; yy = yy+yn; else end
if rot == 4 ss = 3; yy = yn; else end
rot = ss;
end
[Ydec,F2] = freqz(ydec,1,512,2000);
subplot(4,1,3), plot(F2,abs(Ydec)), legend('Spectrum: decimated signal'), ylabel('|Y_d_e_c(F)|')
axis([0,1000,0,60])

% STEP 5
% Interpolation
yi = [0]; xk_i = zeros(size(E));
rot = 0;
k = 1;
for n=1:length(ydec)*5
    xn=ydec(k);
xk = xk_i(rot+1,:); ek = E(rot+1,:); 
xk=[xn,xk(1:length(xk)-1)]; yn=sum(xk.*ek); % filtering
xk_i(rot+1,:)=xk; yn=5*yn;
if rot == 0 ss = 1; yi = [yi,yn]; else end
if rot == 1 ss = 2; yi = [yi,yn]; else end
if rot == 2 ss = 3; yi = [yi,yn]; else end
if rot == 3 ss = 4; yi = [yi,yn]; else end
if rot == 4 ss = 0; k = k+1; yi = [yi,yn]; else end
rot = ss;
end
[Yint,F3] = freqz(yi,1,1024,10000);
subplot(4,1,4)
plot(F3,abs(Yint)); legend('Spectrum: interpolated signal')
xlabel('Frequency, Hz'),ylabel('|Y_i_n_t(F)|')
axis([0,5000,0,300])
