% Program: demo_4_3 
%
%       Created for the book:                                                             
%   Ljiljana D. Milic,
%      Multirate Filtering for Digital Signal Processing: MATLAB Applications                     
%      Information Science Reference, Hershey, New York,  2009
%
% Structure simulation of decimator, Figure 4.13.
% Structure simulation of interpolator, Figure 4.15.
clear all, close all

% STEP 1:  
% Lowpass FIR filter design
h = firpm(11,[0,800/5000,2*1666/5000-800/5000,1],[1,1,0,0]);
[H,F1] = freqz(h,1,1024,10000);
figure (1)
subplot(4,1,1), plot(F1,20*log10(abs(H))), legend('Filter magnitude response'), ylabel('Gain, dB')
axis([0,5000,-60,5])

% STEP 2:  
% Polyphase decomposition
E = reshape(h,3,length(h)/3); E = fliplr(E);

% STEP 3:  
% Input signal
n = 0:510; % Time index
x = cos(2*pi*350*n/10000) + cos(2*pi*750*n/10000) + 0.7*cos(2*pi*3800*n/10000) + randn(size(n));
[X,F1] = freqz(x,1,1024,10000); % Computing the spectrum of the input signal
subplot(4,1,2)
plot(F1,abs(X)), legend('Spectrum: input signal'), ylabel('|X(F)|')
axis([0,5000,0,300])

% STEP 4:  
% Setting the initial states 
xk = zeros(size(1:length(h)/3));
% Decimation
rot = 0; % Initial rotator position
ydec = [];
for n=1:length(x)
xn = x(n); xk = xn*E((rot+1),:)+xk;
if rot == 0 ss = 2; yn = xk(length(xk)); ydec = [yn,ydec]; xk = [0,xk(1:length(xk)-1)]; else end
if rot == 1 ss = 0; else end
if rot == 2 ss = 1; else end
rot = ss; end
[Ydec,F2] = freqz(ydec,1,512,3332);
subplot(4,1,3), plot(F2,abs(Ydec)), legend('Spectrum: decimated signal'), ylabel('|Y_{dec}(F)|')
axis([0,1662,0,100])

% STEP 5:  
% Interpolation
E = 3*fliplr(E);
yi = [];
xk = zeros(size(1:length(h)/3));;
rot = 0; % Initial rotator position
k = 1;
for n=1:length(ydec)*3
    xn = ydec(k); ek = E(rot+1,:);
    if k == 1,  xk = [xn,xk(1:length(xk)-1)]; else end
    yn = sum(xk.*ek); yi = [yi,yn];
if rot == 0 ss = 1; else end                                                                                                                                                                                                                                                                        
if rot == 1 ss = 2; else end
if rot == 2 ss = 0; k = k+1; xk = [xn,xk(1:length(xk)-1)]; else end
rot = ss;
end
[Yint,F3] = freqz(yi,1,1024,10000);
subplot(4,1,4)
plot(F3,abs(Yint)); legend('Spectrum: interpolated signal')
xlabel('Frequency, Hz'),ylabel('|Y_{int}(F)|')
axis([0,5000,0,300])
